@@ -1,15 +1,15 @@
 ;; -*- Mode: Emacs-Lisp -*-
-;;                       $Id: mew-nmz.el,v 1.7.2.1 1999/10/20 11:20:55 kazu Exp $
+;;                       $Id: mew-nmz.el,v 1.7.2.6 2000/02/28 03:52:41 kazu Exp $
 ;;
 ;; mew-nmz.el: Another Mew search method, powered by Namazu.
 ;;
-;;                                "Hideyuki SHIRAI" <Shirai@rdmg.mgcs.mei.co.jp>
+;;                                "Hideyuki SHIRAI" <shirai@rdmg.mgcs.mei.co.jp>
 ;;
 ;;;; Usage: Put your ~/.emacs.
 ;; (eval-after-load "mew" '(require 'mew-nmz))
 ;;
 
-(defconst mew-nmz-version "mew-nmz.el 0.41")
+(defconst mew-nmz-version "mew-nmz.el 0.50")
 
 (eval-when-compile (require 'mew))
 (and (locate-library "namazu")
@@ -21,26 +21,36 @@
 (defvar mew-nmz-index-news "News" "*Namazu index local news directory.")
 (defvar mew-nmz-index-imap "Imap" "*Namazu index IMAP directory.")
 
+(defvar mew-nmz-namazu-version nil
+  "*Automatically set 'v1 if Namazu version 1. Set 'v2 if Namazu version 2.")
+
 (defvar mew-nmz-use-fast-pick t "*Use fast pick mode.")
-(defvar mew-nmz-use-backslash nil "*If t, convert / => \ for Windows")
+(defvar mew-nmz-use-backslash
+  (if (memq system-type '(OS/2 emx windows-nt)) t nil)
+  "*If non-nil, convert / => \ for Windows")
 
 (defvar mew-nmz-prog "namazu" "*Namazu program name.")
-(defvar mew-nmz-prog-args '("-aS") "*Namazu program argument.")
 
 (defvar mew-nmz-db-max 64 "*Namazu max index")
 (defvar mew-nmz-query-max-length 256 "*Namazu query string max length.")
 
 (defvar mew-nmz-prog-mknmz "mknmz" "*Namazu make index program.")
-(defvar mew-nmz-prog-mknmz-args '("-uUhPq") "*mknmz's argment.")
+(defvar mew-nmz-prog-mknmz-args '("-q")
+  "*Mknmz's argment, in addition to \"-U\", \"-h\".")
 (defvar mew-nmz-prog-mknmz-include "~/Namazu/mknmz-inc.pl" "*Include file for mknmz.")
 
 (defvar mew-nmz-prog-gcnmz "gcnmz" "*Namazu refresh index program.")
-(defvar mew-nmz-use-gcnmz-folders '("+inbox")
+(defvar mew-nmz-use-gcnmz-folders (list mew-inbox-folder)
   "*Exec gcnmz after mknmz for index refresh, 't' means all folders.")
 
 (defvar mew-nmz-mknmz-skip-folders
-  '("+trash" "+draft" "+draft/mime" "=draft" "=draft/draft")
-  "*Skip folders, when make index.")
+  (if (and (boundp 'mew-draft-folder) (boundp 'mew-trash-folder)
+	   (boundp 'mew-queue-folder) (boundp 'mew-attach-folder)
+	   mew-draft-folder mew-trash-folder mew-queue-folder mew-attach-folder)
+      (list mew-draft-folder mew-trash-folder mew-queue-folder mew-attach-folder
+	    "+schedule" "=draft")
+    (list "+draft" "+trash" "+schedule" "=draft"))
+  "*Skip folders regexp, when make index.")
 (defvar mew-nmz-mknmz-skip-news t "*Skip local news folders, when make index.")
 (defvar mew-nmz-mknmz-use-mode-line t "*Display \"nmz\" in mode line , when mknmzing.")
 (defvar mew-nmz-line-id '("Mew(nmz): %7b")
@@ -55,7 +65,7 @@ A good example is \"+from:\".")
   "*A list of key for namazu pick pattern.")
 
 (defvar mew-nmz-imap-localfile-suffix ".gz"
-  "*IMAP local file's suffix.")
+  "*IMAP local file's suffix, need mew-fake-imap.el.")
 
 (defvar mew-nmz-pick-gather-field-list
   (list (list mew-from: 'address "+from:" "+to:" "+cc:")
@@ -81,8 +91,17 @@ A good example is \"+from:\".")
 	     (define-key mew-summary-mode-map "zm" 'mew-nmz-mknmz)
 	     (define-key mew-summary-mode-map "zu" 'mew-nmz-mark-unindexed)
 	     (define-key mew-summary-mode-map "z^" 'mew-nmz-search-parent)
+	     (define-key mew-summary-mode-map "zp" 'mew-nmz-search-parent)
+	     (define-key mew-summary-mode-map "zn" '(lambda (arg)
+						      (interactive "P")
+						      (mew-nmz-search-parent t)))
 	     (define-key mew-summary-mode-map "zN" 'mew-nmz-namazu)))
 
+(add-hook 'mew-message-mode-hook
+	  '(lambda ()
+	     (define-key mew-message-mode-map "zp" 'mew-nmz-search-msgid-at-point)
+	     (define-key mew-message-mode-map "zr" 'mew-nmz-search-msgid-region)))
+
 (add-hook 'mew-quit-hook
 	  '(lambda ()
 	     (setq mew-nmz-indexed-folders nil)))
@@ -120,6 +139,8 @@ If executed with 'C-u', jump to original message folder and number."
 (defvar mew-nmz-gather-header-list nil)
 (defvar mew-nmz-namazu-last-folder nil)
 (defvar mew-nmz-indexed-folders nil)
+(defvar mew-nmz-namazu-version1-str "^  Search Program of Namazu v1\.[34]\.")
+(defvar mew-nmz-namazu-version2-str "^namazu of Namazu [1-9.]+")
 
 ;; macros
 (defmacro mew-nmz-expand-folder (folder)
@@ -169,6 +190,29 @@ If executed with 'C-u', jump to original message folder and number."
 	 (mew-nmz-dir-to-folder-imap (substring (, nmzdir) (match-beginning 1) (match-end 1))))
 	(t nil)))))
 
+;;
+;; Namazu Version check.
+(defun mew-nmz-version-set ()
+  (interactive)
+  (if mew-nmz-namazu-version
+      ()
+    (mew-set-buffer-tmp)
+    (mew-piolet
+     mew-cs-autoconv mew-cs-pick
+     (apply (function call-process)
+	    mew-nmz-prog nil t nil (list "-L" "en" "-v")))
+    (goto-char (point-min))
+    (if (re-search-forward mew-nmz-namazu-version2-str nil t)
+	(setq mew-nmz-namazu-version 'v2)
+      (goto-char (point-min))
+      (if (re-search-forward mew-nmz-namazu-version1-str nil t)
+	  (setq mew-nmz-namazu-version 'v1)
+	(ding)
+	(message "Something error occor. (Namazu version check)")
+	(sit-for 1)))))
+
+(add-hook 'mew-init-hook 'mew-nmz-version-set)
+
 ;;
 ;; "Make Index" functions.
 (defun mew-nmz-mknmz (&optional folder remove callp)
@@ -182,13 +226,14 @@ If executed with 'C-u', remove index files at the specified folder."
       (if (not (mew-which mew-nmz-prog-mknmz exec-path))
 	  (message "Please install mknmz.")
 	(and current-prefix-arg (setq remove t))
-	(and (not folder)
-	     (setq folder (mew-input-folder (buffer-name))))
+	(if  (not folder)
+	    (setq folder (mew-input-folder (buffer-name)))
+	  (setq folder (directory-file-name folder)))
 	(if (mew-folder-imapp folder)
 	    (setq suffix mew-nmz-imap-localfile-suffix))
 	(if (or (mew-folder-newsp folder)
 		(mew-folder-virtualp folder)
-		(member folder mew-nmz-mknmz-skip-folders)
+		(mew-nmz-skip-folder folder)
 		(and mew-nmz-mknmz-skip-news
 		     (string-match "^=" folder)))
 	    (and msgenb (message "Can't make namazu index in %s." folder))
@@ -200,14 +245,18 @@ If executed with 'C-u', remove index files at the specified folder."
 		(and msgenb (message "Detect running mknmz process in %s." folder))
 	      (if (and folder-dir namazu-dir
 		       (file-directory-p folder-dir))
-		  (let ((file-list (directory-files folder-dir nil
+		  (let ((file-list (directory-files folder-dir 'full-name
 						    (concat "^[1-9][0-9]*" suffix "$")
-						    nil))
+						    'no-sort))
 			(temp-file (expand-file-name
 				    (mew-nmz-make-temp-name "mknmz_") mew-temp-dir))
 			(prog-args mew-nmz-prog-mknmz-args)
 			(exist-msg nil)
 			file)
+		    (if (and (eq 'v2 mew-nmz-namazu-version)
+			     (not (string= suffix "")))
+			(setq prog-args (append prog-args (list "-U")))
+		      (setq prog-args (append prog-args (list "-Uh"))))
 		    (while (and mew-nmz-use-backslash (file-exists-p temp-file))
 		      (message "Warning!! same name det.")
 		      (sit-for 1)
@@ -217,11 +266,11 @@ If executed with 'C-u', remove index files at the specified folder."
 		    (while file-list
 		      (setq file (car file-list))
 		      (setq exist-msg t)
-		      (if mew-nmz-use-backslash
-			  (insert (mew-nmz-slash-to-backslash
-				   (expand-file-name file folder-dir))
-				  "\n")
-			(insert (expand-file-name file folder-dir) "\n"))
+		      (if (not (file-directory-p file))
+			  (if (and mew-nmz-use-backslash
+				   (eq mew-nmz-namazu-version 'v1))
+			      (insert (mew-nmz-slash-to-backslash file) "\n")
+			    (insert file "\n")))
 		      (setq file-list (cdr file-list)))
 		    (mew-frwlet
 		     mew-cs-autoconv mew-cs-pick
@@ -244,7 +293,7 @@ If executed with 'C-u', remove index files at the specified folder."
 		      (setq prog-args (append prog-args
 					      (list "-F" temp-file
 						    "-O" namazu-dir)))
-		      (if (file-name-all-completions "NMZ.i." namazu-dir)
+		      (if (file-name-all-completions "NMZ.lock" namazu-dir)
 			  (progn
 			    (message "Warning!! Something error in %s's index." folder)
 			    (sit-for 1)))
@@ -280,7 +329,16 @@ If executed with 'C-u', remove index files at the specified folder."
 					  nil
 					  (current-buffer)
 					  nil
-					  (list (expand-file-name "NMZ" namazu-dir))))))
+					  (list
+					   (and (eq mew-nmz-namazu-version 'v2)
+						"--no-backup")
+					   (expand-file-name "NMZ" namazu-dir)))
+				   (if (eq mew-nmz-namazu-version 'v1)
+				       (let ((backfn (directory-files namazu-dir t ".*.BAK$")))
+					 (while backfn
+					   (if (file-writable-p (car backfn))
+					       (delete-file (car backfn)))
+					   (setq backfn (cdr backfn))))))))
 			    (delete-file temp-file)
 			    (goto-char (point-min))
 			    (if (search-forward-regexp "^ERROR:.*$" nil t)
@@ -325,9 +383,7 @@ If executed with 'C-u', remove index files at the specified folder."
 		(setq msg (format "Mew mknmz (%s) ... %s." folder (mew-match 0)))
 	      (setq msg (format "Mew mknmz (%s) ... done" folder))
 	      (if (not (or (null mew-nmz-indexed-folders)
-			   (member folder mew-nmz-indexed-folders)
-			   (member (concat folder (regexp-quote mew-path-separator))
-				   mew-nmz-indexed-folders)))
+			   (member folder mew-nmz-indexed-folders)))
 		  (setq mew-nmz-indexed-folders
 			(cons folder mew-nmz-indexed-folders))))
 	    (and (file-readable-p tmpfile)
@@ -384,20 +440,24 @@ If executed with 'C-u', remove index files at the specified folder."
 	 (if (and (mew-summary-mark-collect
 		   mew-nmz-mark-unindexed (point-min) (point-max))
 		  (y-or-n-p (format "Unmark '%c' ? " mew-nmz-mark-unindexed)))
-	     (mew-summary-batch-unmark (list mew-nmz-mark-unindexed) 'msg))
-	 (let* ((url-file-name (expand-file-name
-				"NMZ.field.url"
-				(mew-nmz-expand-folder (buffer-name))))
+	     (if (fboundp 'mew-summary-batch-unmark) ;; 1.94.2
+		 (mew-summary-batch-unmark (list mew-nmz-mark-unindexed) 'msg)
+	       (mew-mark-undo-marks (list mew-nmz-mark-unindexed))))
+	 (let* ((url-file-name
+		 (expand-file-name
+		  (if (eq 'v2 mew-nmz-namazu-version)
+		      "NMZ.field.uri" "NMZ.field.url")
+		  (mew-nmz-expand-folder (buffer-name))))
 		(marked-messages 0)
 		(unindexed-messages 0)
 		(suffix (if (mew-folder-imapp (buffer-name))
 			    mew-nmz-imap-localfile-suffix ""))
 		msgnums)
-	   (if (not (file-readable-p url-file-name))
+	   (if (not (file-exists-p url-file-name))
 	       (message "%s has no index file." (buffer-name))
 	     (save-excursion
 	       (mew-set-buffer-tmp)
-	       (message "checking NMZ.filed.url...")
+	       (message "checking %s..." (file-name-nondirectory url-file-name))
 	       (insert-file-contents url-file-name)
 	       (while (search-forward-regexp
 		       (concat "/\\([1-9][0-9]*\\)" suffix "$") nil t)
@@ -520,6 +580,16 @@ If executed with 'C-u', search *child* message."
 	(mew-nmz-goto-folder-msg folder (car range))
 	(message "Searching %s ... %s/%s." message folder (car range))))))
 
+(defun mew-nmz-search-msgid-at-point ()
+  (interactive)
+  (let (start end (pos (point)))
+    (if (and (re-search-backward "<" (save-excursion (beginning-of-line) (point)) t)
+	     (setq start (point))
+	     (re-search-forward ">" (save-excursion (end-of-line) (point)) t)
+	     (setq end (point)))
+	(mew-nmz-search-msgid (buffer-substring start end))
+      (message "No Message-ID."))))
+
 (defun mew-nmz-search-msgid-region (start end)
   (interactive "r")
   (mew-nmz-search-msgid (buffer-substring start end)))
@@ -527,7 +597,8 @@ If executed with 'C-u', search *child* message."
 (defun mew-nmz-search-msgid (mid)
   (interactive "sMessage-ID: ")
   (if (string-match "\\(<[^>]+>\\)" mid)
-      (let ((mew-window-use-full t))
+      (let ((mew-window-use-full t)
+	    (mew-use-full-window t))
 	(mew-nmz-search-parent nil (mew-match 1 mid)))
     (message "No Message-ID.")))
 
@@ -590,9 +661,11 @@ then list them up used."
        (message "Namazu picking messages in %s ..." folder)
        (setq range (mew-nmz-pick folder pattern))
        (message "Namazu picking messages in %s ... done" folder)
-       (if (get-buffer folder)
-	   (switch-to-buffer folder)
-	 (mew-summary-folder-create folder))
+       (if (fboundp 'mew-summary-folder-create)  ;; 1.94.2
+	   (if (get-buffer folder)
+	       (switch-to-buffer folder)
+	     (mew-summary-folder-create folder))
+	 (mew-summary-switch-to-folder folder))
        (if range
 	   (if (and (not arg)
 		    mew-nmz-use-fast-pick
@@ -640,9 +713,7 @@ then list them up used."
 		 ((string-match "^.*\\*$" folder)
 		  (setq namazu-dirs (append namazu-dirs
 					    (mew-nmz-expand-folder-regexp folder))))
-		 ((or (member folder mew-nmz-indexed-folders)
-		      (member (concat folder (regexp-quote mew-path-separator))
-			      mew-nmz-indexed-folders))
+		 ((member folder mew-nmz-indexed-folders)
 		  (setq namazu-dirs (cons (mew-nmz-expand-folder folder) namazu-dirs)))))
 	     folders)
      (setq namazu-dirs (mew-uniq-list namazu-dirs))
@@ -711,11 +782,18 @@ then list them up used."
 		       (insert str)
 		       (setq str "")))
 		   (setq fld-msgs (cdr fld-msgs))))
-	       (mew-summary-batch-unmark (list mew-mark-multi
-					       mew-mark-review
-					       mew-mark-delete
-					       mew-mark-refile
-					       mew-mark-tmp) nil)))
+	       (if (fboundp 'mew-summary-batch-unmark) ;; 1.94.2
+		   (mew-summary-batch-unmark (list mew-mark-multi
+						   mew-mark-review
+						   mew-mark-delete
+						   mew-mark-refile
+						   mew-mark-tmp) nil)
+		 (mew-mark-undo-marks (list mew-mark-multi
+					    mew-mark-review
+					    mew-mark-delete
+					    mew-mark-unlink
+					    mew-mark-refile
+					    mew-mark-tmp) 'nomsg))))
 	 (goto-char (point-min))
 	 (setq buffer-read-only t)
 	 (if (null namazu-im-dirs)
@@ -725,8 +803,8 @@ then list them up used."
 		 (message "Listing %s ..." vfolder)
 		 (setq mew-summary-buffer-start-point (point))
 		 (setq mew-summary-buffer-string nil) ;; just in case
-		 (mew-piolet
-		  mew-cs-virtual mew-cs-pick
+		 (mew-pioalet
+		  mew-cs-virtual mew-cs-pick mew-cs-pick
 		  (setq mew-summary-buffer-process
 			;; imls --namazu=yes --grep=pattern --src=namazu's index,... number
 			(apply (function start-process)
@@ -767,9 +845,9 @@ then list them up used."
 	  '(lambda ()
 	     (define-key namazu-mode-map "m" 'mew-nmz-namazu-goto-mew)
 	     (define-key namazu-mode-map "M" 'mew-nmz-namazu-return-mew)
-	     (if (string-match "XEmacs" emacs-version)
-		 (define-key namazu-mode-map [(button2)] 'mew-nmz-namazu-view-at-mouse)
-	       (define-key namazu-mode-map [mouse-2] 'mew-nmz-namazu-view-at-mouse))))
+	     (if (featurep 'xemacs)
+		 (define-key namazu-mode-map [(shift button2)] 'mew-nmz-namazu-view-at-mouse)
+	       (define-key namazu-mode-map [S-mouse-2] 'mew-nmz-namazu-view-at-mouse))))
 
 (defun mew-nmz-namazu (&optional arg)
   "Use namazu-mode from mew."
@@ -789,9 +867,7 @@ then list them up used."
 		       ((string-match "^.*\\*$" folder)
 			(setq namazu-dirs (append namazu-dirs
 						  (mew-nmz-expand-folder-regexp folder))))
-		       ((or (member folder mew-nmz-indexed-folders)
-			    (member (concat folder (regexp-quote mew-path-separator))
-				    mew-nmz-indexed-folders))
+		       ((member folder mew-nmz-indexed-folders)
 			(setq namazu-dirs (cons (mew-nmz-expand-folder folder) namazu-dirs)))))
 		   folders)
 	   (setq namazu-dirs (mew-uniq-list namazu-dirs))
@@ -799,21 +875,25 @@ then list them up used."
 	   (if (not (> dirlen mew-nmz-db-max))
 	       ()
 	     (message "Warning: assigned indexes over DB_MAX.")
-	     (sit-for 1)
-	     (setq dirlen (- dirlen mew-nmz-db-max))
-	     (setq namazu-dirs (nreverse (nthcdr dirlen namazu-dirs))))
+	     (sit-for 1))
+	   ;; (setq dirlen (- dirlen mew-nmz-db-max))
+	   ;; (setq namazu-dirs (nreverse (nthcdr dirlen namazu-dirs))))
 	   (and (not (fboundp 'namazu))
 		(require 'namazu))
 	   (namazu 0 (mew-join " " namazu-dirs) grep)
-	   (setq namazu-history (cons grep namazu-history))
+	   (cond
+	    ((boundp 'namazu-history)
+	     (setq namazu-history (cons grep namazu-history)))
+	    ((boundp 'namazu-keyword-history)
+	     (setq namazu-keyword-history (cons grep namazu-keyword-history))))
 	   (namazu-jump-next)
 	   (if mew-nmz-namazu-full-window (delete-other-windows)))))))
 
 (defun mew-nmz-namazu-goto-mew ()
   (interactive)
-  (let ((pattern (concat "^\\(/.*\\)/\\([1-9][0-9]*\\)\\("
+  (let ((pattern (concat "^\\(~?/.*\\)/\\([1-9][0-9]*\\)\\("
 			 mew-nmz-imap-localfile-suffix
-			 " \\| \\)size .*$"))
+			 " \\| \\).*$"))
 	fld msg)
     (beginning-of-line)
     (if (not (re-search-forward pattern nil t))
@@ -845,29 +925,29 @@ then list them up used."
   (set-buffer (event-buffer event))
   (goto-char (event-point event))
   (let ((pos (point))
-	(mew-mail-pattern (concat "^"
-				  (expand-file-name mew-mail-path)
-				  ".*/[1-9][0-9]*"))
-	(mew-news-pattern (concat "^"
-				  (expand-file-name mew-news-path)
-				  ".*/[1-9][0-9]*"))
+	(mew-mail-pattern
+	 (concat "^\\("
+		 (if (not mew-nmz-use-backslash)
+		     (expand-file-name mew-mail-path)
+		   (concat "/"
+			   (substring (expand-file-name mew-mail-path) 0 1)
+			   "|"
+			   (substring (expand-file-name mew-mail-path) 2)))
+		 "\\|~/Mail\\)/.*/[1-9][0-9]*\\("
+		 mew-nmz-imap-localfile-suffix
+		 "\\)?"))
+	(mew-news-pattern
+	 (concat "^\\("
+		 (if (not mew-nmz-use-backslash)
+		     (expand-file-name mew-news-path)
+		   (concat "/"
+			   (substring (expand-file-name mew-news-path) 0 1)
+			   "|"
+			   (substring (expand-file-name mew-news-path) 2)))
+		 "\\|~/News\\)/.*/[1-9][0-9]*\\("
+		 mew-nmz-imap-localfile-suffix
+		 "\\)?"))
 	pos-title pos-url)
-    (and mew-nmz-use-backslash
-	 (string-match "^\\^\\([a-zA-Z]\\):\\(/.*\\)" mew-mail-pattern)
-	 (setq mew-mail-pattern
-	       (concat
-		"^/"
-		(substring mew-mail-pattern (match-beginning 1) (match-end 1))
-		"|"
-		(substring mew-mail-pattern (match-beginning 2) (match-end 2)))))
-    (and mew-nmz-use-backslash
-	 (string-match "^\\^\\([a-zA-Z]\\):\\(/.*\\)" mew-news-pattern)
-	 (setq mew-news-pattern
-	       (concat
-		"^/"
-		(substring mew-news-pattern (match-beginning 1) (match-end 1))
-		"|"
-		(substring mew-news-pattern (match-beginning 2) (match-end 2)))))
     (end-of-line)
     (and (re-search-backward namazu-output-title-pattern nil t)
 	 (setq pos-title (point))
@@ -994,11 +1074,13 @@ then list them up used."
 	  (if last (setq last (string-to-int last)))
 	  (mew-set-buffer-tmp)
 	  (buffer-disable-undo (current-buffer))
-	  (setq prog-args (append mew-nmz-prog-args
+	  (setq prog-args (append (if (eq mew-nmz-namazu-version 'v1)
+				      (list "-aeS")
+				    (list "--all" "--list" "--early"))
 				  (list pattern)
 				  (list namazu-dir)))
-	  (mew-piolet
-	   mew-cs-autoconv mew-cs-pick
+	  (mew-pioalet
+	   mew-cs-autoconv mew-cs-pick mew-cs-pick
 	   (apply (function call-process)
 		  mew-nmz-prog nil t nil prog-args))
 	  (goto-char (point-min))
@@ -1034,11 +1116,13 @@ then list them up used."
 		      (and catch (null intmsgs))))
 	(mew-set-buffer-tmp)
 	(buffer-disable-undo (current-buffer))
-	(setq prog-args (append mew-nmz-prog-args
+	(setq prog-args (append (if (eq mew-nmz-namazu-version 'v1)
+				    (list "-aS")
+				  (list "--all" "--list"))
 				(list pattern)
 				(car nmzdirs)))
-	(mew-piolet
-	 mew-cs-autoconv mew-cs-pick
+	(mew-pioalet
+	 mew-cs-autoconv mew-cs-pick mew-cs-pick
 	 (apply (function call-process)
 		mew-nmz-prog nil t nil prog-args))
 	(goto-char (point-min))
@@ -1118,12 +1202,13 @@ then list them up used."
 
 (defun mew-nmz-url-to-folder (url)
   "Convert namazu's output url to folder."
-  (if (and mew-nmz-use-backslash
-	   (string-match "^/\\([a-zA-Z]\\)|\\(/.*\\)" url))
-      (setq url (concat
-		 (substring url (match-beginning 1) (match-end 1))
-		 ":"
-		 (substring url (match-beginning 2) (match-end 2)))))
+  (and mew-nmz-use-backslash
+       (string-match "^/\\([a-zA-Z]\\)|\\(/.*\\)" url)
+       (setq url (concat
+		  (substring url (match-beginning 1) (match-end 1))
+		  ":"
+		  (substring url (match-beginning 2) (match-end 2)))))
+  (setq url (expand-file-name url))
   (cond
    ((string-match (concat
 		   "^"
@@ -1146,7 +1231,9 @@ then list them up used."
 		   "\\(.*\\)$")
 		  url)
     (concat "=" (substring url (match-beginning 1) (match-end 1))))
-   (t nil)))
+
+   (t (message "Ignore url(%s) in %s." url
+	       (if (eq mew-nmz-namazu-version 'v1) "NMZ.field.url" "NMZ.field.uri")))))
 
 (defun mew-nmz-expand-folder-regexp (folder)
   "Convert folder to namazu-index-dir with '*' expand."
@@ -1159,11 +1246,11 @@ then list them up used."
 	    (setq folder "^[+=%]")
 	  (setq folder "^[+=]"))
       (setq folder (mew-match 1 folder))
-      (setq folder (concat "^" (regexp-quote folder))))
+      (if (string-match "^\\(.*\\)/$" folder)
+	  (setq folder (concat "^" (regexp-quote (mew-match 1 folder))))
+	(setq folder (concat "^" (regexp-quote folder)))))
     (while (setq fld (car nmz-fld-list))
-      (and (or (string-match folder fld)
-	       (string-match (concat folder (regexp-quote mew-path-separator))
-			     fld))
+      (and (string-match folder fld)
 	   (setq namazu-dir-list
 		 (cons (mew-nmz-expand-folder fld) namazu-dir-list)))
       (setq nmz-fld-list (cdr nmz-fld-list)))
@@ -1178,7 +1265,8 @@ then list them up used."
 	       (and (setq nmzdir (mew-nmz-expand-folder fld))
 		    (file-directory-p nmzdir)
 		    (file-exists-p (expand-file-name "NMZ.i" nmzdir))
-		    (setq namazu-folder-list (cons fld namazu-folder-list))))
+		    (setq namazu-folder-list
+			  (cons (directory-file-name fld) namazu-folder-list))))
 	    folder-list)
     (setq mew-nmz-indexed-folders (nreverse namazu-folder-list))
     (and (interactive-p) (message "Gather indexed folder...done."))))
@@ -1187,15 +1275,29 @@ then list them up used."
   "Delete namazu index files."
   (let ((nmz-list (file-name-all-completions "NMZ\." namazu-dir)))
     (mapcar '(lambda (file)
-	       (delete-file (expand-file-name file namazu-dir)))
+	       (and (file-writable-p (expand-file-name file namazu-dir))
+		    (delete-file (expand-file-name file namazu-dir))))
 	    nmz-list)))
 
+(defun mew-nmz-skip-folder (fld)
+  (let ((fls mew-nmz-mknmz-skip-folders))
+    (catch 'match
+      (while fls
+	(if (string-match (concat "^" (car fls)) fld)
+	    (throw 'match t))
+	(setq fls (cdr fls)))
+      nil)))
+
 (defun mew-nmz-folder-dir-newp (namazu-dir)
   (let* ((folder (mew-nmz-dir-to-folder namazu-dir))
 	 (buf (get-buffer folder)))
     (if (not buf)
 	(let* ((dir (file-chase-links (mew-expand-folder folder)))
-	       (tdir (nth 5 (file-attributes dir)))
+	       (tdir (if (and mew-touch-folder-p (boundp 'mew-summary-touch-file))
+			 (nth 5 (file-attributes
+				 (expand-file-name mew-summary-touch-file
+						   (mew-expand-folder dir))))
+		       (nth 5 (file-attributes dir))))
 	       (da (car tdir))
 	       (db (car (cdr tdir)))
 	       (cache (expand-file-name mew-summary-cache-file dir))
