@@ -3,11 +3,11 @@
 ;; Author:  Yoshinari NOMURA <nom@csce.kyushu-u.ac.jp>
 ;;          Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Jun 11, 1994
-;; Revised: Sep  1, 1998
+;; Revised: Aug 30, 1999
 
 ;;; Code:
 
-(defconst mew-refile-version "mew-refile.el version 0.65")
+(defconst mew-refile-version "mew-refile.el version 0.76")
 
 (require 'mew)
 
@@ -16,30 +16,19 @@
 ;;; Variables
 ;;;
 
-(defvar mew-refile-msgid-alist nil)
+(defvar mew-refile-msgid-alist nil
+  "Alist of message-id and folder pair")
 (defvar mew-refile-msgid-file-name ".mew-refile-msgid-alist")
-(defvar mew-refile-from-alist nil)
+(defvar mew-refile-from-alist nil
+  "Alist of From: address and folder pair")
 (defvar mew-refile-from-file-name ".mew-refile-from-alist")
 
-(defvar mew-refile-alist-max-length 1000
-  "*Max length of mew-refile-from-alist and mew-refile-msgid-alist.")
-
 (defvar mew-refile-last-folder nil
   "Folder name previously you refiled")
 
 (defvar mew-refile-ctrl-multi t
   "*If *non-nil*, guess functions guess multi folders.")
 
-(defvar mew-refile-ctrl-throw nil
-  "*If *non-nil*, guess function doesn't guess any more if some other
-functions were successful. This variable is set in guess control
-functions. So this is not a user customize variable.")
-
-(defvar mew-refile-ctrl-auto-boundary nil
-  "*If *non-nil*, auto refile function stops at this point. This variable
-is set in guess control functions. So this is not a user customize
-variable.")
-
 (defvar mew-refile-guess-alist nil
   "*If non-nil, mew guesses destination folder by using this hint.
 The format is like this:
@@ -61,30 +50,44 @@ The format is like this:
 
 (defvar mew-refile-guess-control
   '(mew-refile-guess-by-alist
+    mew-refile-ctrl-throw
     mew-refile-guess-by-newsgroups
     mew-refile-guess-by-folder
     mew-refile-ctrl-throw
     mew-refile-ctrl-auto-boundary
-    mew-refile-guess-by-msgid
+    mew-refile-guess-by-thread
+    mew-refile-ctrl-throw
+    mew-refile-guess-by-from-folder
+    mew-refile-ctrl-throw
     mew-refile-guess-by-from
+    mew-refile-ctrl-throw
     mew-refile-guess-by-default))
 
 (defvar mew-refile-auto-refile-skip-any-mark nil
-  "*If *non-nil*, mew-summary-auto-refile doesn't touch
+  "*If *non-nil*, 'mew-summary-auto-refile' doesn't touch
 any alredy marked message.")
 
+(defvar mew-refile-auto-refile-confirm nil
+  "*If *non-nil*, 'mew-summary-auto-refile' prompts the user for
+confirmation before refiling.")
+
+(defvar mew-refile-guess-strip-domainpart t
+  "*If *non-nil*, 'mew-refile-guess-by-default' strips domainpart of from")
+
+(defvar mew-refile-guess-from-me-is-special nil
+  "*If *non-nil*, 'mew-refile-guess-by-from-*' think the mails from yourself
+as special. They use To: or Cc: instead of From:")
+
 ;;
 ;; initialize function
 ;;
 (defun mew-refile-init ()
   ;; load message id alist
   (or mew-refile-msgid-alist
-      (setq mew-refile-msgid-alist
-	    (mew-refile-alist-load mew-refile-msgid-file-name)))
+      (setq mew-refile-msgid-alist (mew-lisp-load mew-refile-msgid-file-name)))
   ;; load from alist
   (or mew-refile-from-alist
-      (setq mew-refile-from-alist
-	    (mew-refile-alist-load mew-refile-from-file-name))))
+      (setq mew-refile-from-alist (mew-lisp-load mew-refile-from-file-name))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -104,48 +107,42 @@ any alredy marked message.")
 ;; that is, 'car' is a list of judged  folders.
 ;;          'cdr' is a alist of opinions by guess functions.
 ;;
-(defun mew-refile-guess (&optional auto)
-  (let ((funcs mew-refile-guess-control) ret guess info)
-    (mew-refile-ctrl-throw-off)
-    (mew-refile-ctrl-auto-boundary-off)
-    (while funcs
-      (if (string-match "ctrl" (symbol-name (car funcs)))
-	  ;; func is control function
-	  (funcall (car funcs))
-	;; func is guess function
-	(setq ret (funcall (car funcs))
-	      ret (or (and (listp ret) ret)
-		      (and ret (list ret))))
-	(if (not (or (and guess mew-refile-ctrl-throw)
-		     (and auto mew-refile-ctrl-auto-boundary)))
-	    (setq guess (append guess ret)))
-	(setq info (cons (cons (car funcs) ret) info)))
-      (setq funcs (cdr funcs)))
-    (setq info (nreverse info))
+(defun mew-refile-guess (&optional auto show-all)
+  (let ((funcs mew-refile-guess-control) ret guess info stop)
+    (catch 'last
+      (while funcs
+	(cond
+	 ;; func is control function
+	 ((string-match "ctrl" (symbol-name (car funcs)))
+	  (if (setq ret (funcall (car funcs) guess auto))
+	      (progn
+		(setq stop t)
+		(or show-all (throw 'last t)))))
+	 ;; func is guess function
+	 (t
+	  (setq ret (funcall (car funcs)))))
+	;; now, ret is the return value from a function.
+	;; make sure ret is a list.
+	(setq ret (or (and (listp ret) ret) (and ret (list ret))))
+	;; shape the guess: (guess1 guess2 ...)
+	;;            info: (info1 info2 ...)
+	(setq info (nconc info (list (cons (car funcs) ret))))
+	(if (not stop) (setq guess (nconc (reverse ret) guess)))
+	(setq funcs (cdr funcs))))
+    (setq guess (nreverse guess))
     (if mew-refile-ctrl-multi
 	(cons (mew-uniq-list guess) info)
       (cons (list (car guess)) info))))
 
+
 ;;
 ;; guess control functions
 ;;
-(defun mew-refile-ctrl-auto-boundary ()
-  (setq mew-refile-ctrl-auto-boundary t))
+(defun mew-refile-ctrl-auto-boundary (guess auto)
+  (if auto 'stop))
 
-(defun mew-refile-ctrl-auto-boundary-on ()
-  (setq mew-refile-ctrl-auto-boundary t))
-
-(defun mew-refile-ctrl-auto-boundary-off ()
-  (setq mew-refile-ctrl-auto-boundary nil))
-
-(defun mew-refile-ctrl-throw ()
-  (setq mew-refile-ctrl-throw t))
-
-(defun mew-refile-ctrl-throw-on ()
-  (setq mew-refile-ctrl-throw t))
-
-(defun mew-refile-ctrl-throw-off ()
-  (setq mew-refile-ctrl-throw nil))
+(defun mew-refile-ctrl-throw (guess auto)
+  (if guess 'stop))
 
 ;;
 ;; by alist returns: (guess1 guess2 ...) or nil
@@ -177,28 +174,32 @@ any alredy marked message.")
 		    (setq ent (mew-refile-guess-by-alist1 val)))))
 	      (if ent
 		  (if (listp ent)
-		      (setq ret (append ent ret))
+		      (setq ret (nconc ent ret) ent nil)
 		    (setq ret (cons ent ret))))
 	      (setq sublist (cdr sublist))))))
       (setq alist (cdr alist)))
     (mew-uniq-list (nreverse ret))))
 
 (defun mew-refile-guess-by-alist2 (regexp field string)
-  (let ((p 1) match-list b e)
+  (let (match-strings match-list)
     (string-match regexp field)
-    (while (<= p 9)
-      (setq b (or (match-beginning p) 0))
-      (setq e (or (match-end p) 0))
-      (setq match-list (cons (substring field b e) match-list))
-      (setq p (1+ p)))
-    (setq  p 1)
-    (while (<= p 9)
-      (if (string-match (concat "\\\\" (int-to-string p)) string)
-	  (setq string
-		(concat (substring string 0 (match-beginning 0))
-			(nth (- 9 p) match-list)
-			(substring string (match-end 0))))
-	(setq p (1+ p))))
+    (setq match-list (cdr (cdr (match-data))))
+    (while (car match-list)
+      (setq match-strings
+	    (cons (substring field
+			     (car match-list) (car (cdr match-list)))
+		  match-strings))
+      (setq match-list (cdr (cdr match-list))))
+    (while (string-match "\\\\\\([1-9]\\)" string)
+	(setq string
+	      (concat (substring string 0 (match-beginning 0))
+		      (nth (- (length match-strings)
+			      (string-to-int 
+			       (substring string
+					  (match-beginning 1)
+					  (match-end 1))))
+			   match-strings)
+		      (substring string (match-end 0)))))
     string))
 
 ;;
@@ -225,17 +226,14 @@ any alredy marked message.")
     (while to-cc
       (setq ml-name (mew-addrstr-extract-user (or (car to-cc) "")))
       (setq ent (mew-assoc-case-equal ml-name mew-folder-alist 1))
-      (if (not ent)
-	  ()
-	(setq ret (cons (nth 0 ent) ret))
-	(mew-refile-treat-alias-insert (car to-cc)))
+      (if ent (setq ret (cons (nth 0 ent) ret)))
       (setq to-cc (cdr to-cc)))
     (mew-uniq-list (nreverse ret))))
 
 ;;
-;; by message-id returns: guess1 or nil
+;; by message-thread returns: guess1 or nil
 ;;
-(defun mew-refile-guess-by-msgid ()
+(defun mew-refile-guess-by-thread ()
   (let ((msgid (or (mew-header-get-value mew-references:)
 		   (mew-header-get-value mew-in-reply-to:))))
     ;; search for msgid
@@ -247,48 +245,66 @@ any alredy marked message.")
 		      mew-refile-msgid-alist)))))
 
 ;;
-;; by from returns: guess1
+;; by from-{folder,alist} returns: guess1 or (guess1 ...) or nil
 ;;
+(defun mew-refile-guess-by-from-folder (&optional addr)
+  (mew-refile-guess-from-dispatch 'mew-refile-guess-by-from-folder-body addr))
+
 (defun mew-refile-guess-by-from (&optional addr)
-  (let ((from (or addr (mew-header-parse-address mew-from:) "")) default)
-    ;; search from alist
-    (setq default (mew-refile-guess-by-default from))
-    (if (file-exists-p (mew-expand-folder default))
-	default
-      (cdr (assoc from mew-refile-from-alist)))))
+  (mew-refile-guess-from-dispatch 'mew-refile-guess-by-from-body addr))
+
+(defun mew-refile-guess-by-from-folder-body (&optional addr)
+  (let* ((pfix (cond ((equal mew-folders-default-folder "" ) "+")
+		     ((equal mew-folders-default-folder nil) "+")
+		     ((equal mew-folders-default-folder "+") "+")
+		     (t (file-name-as-directory mew-folders-default-folder))))
+	 (from (downcase (or addr (mew-header-parse-address mew-from:) "")))
+	 (user (mew-addrstr-extract-user from))
+	 (pfix-regex (concat "^" (regexp-quote pfix) "\\(.*/\\)?"))
+	 (from-regex (concat pfix-regex (regexp-quote from) "/?$"))
+	 (user-regex (concat pfix-regex (regexp-quote user) "/?$")))
+    (or
+     (mew-refile-match-in-list from-regex mew-folder-list)
+     (mew-refile-match-in-list user-regex mew-folder-list))))
+
+(defun mew-refile-guess-by-from-body (&optional addr)
+  (let ((from   (downcase (or addr (mew-header-parse-address mew-from:) ""))))
+    (cdr (assoc from mew-refile-from-alist))))
 
 ;;
-;; by To: or Cc: when From: is mine. (Undocumented)
+;; dispatcher to make mew-refile-guess-by-from-* consider
+;; mew-refile-guess-from-me-is-special.
 ;;
-(defun mew-refile-guess-by-from2 ()
-  (let ((to-cc (mew-header-parse-address-list mew-refile-guess-key-list))
-	(from  (mew-header-parse-address mew-from:)))
-    (if (catch 'my-addr
-	  (mapcar (lambda (a)
-		    (if (string-match a from)
-			(throw 'my-addr t)))
-		  (cons (concat "^" (regexp-quote (user-login-name)) "$")
-			(cons (concat "^" mew-mail-address "$")
-			      mew-mail-address-list)))
-	  nil)
-	(let (ent ret)
-	  (while to-cc
-	    (setq ent (mew-refile-guess-by-from (car to-cc)))
-	    (if ent (setq ret (cons ent ret)))
-	    (setq to-cc (cdr to-cc)))
-	  (mew-uniq-list (nreverse ret)))
-      ())))
+(defun mew-refile-guess-from-dispatch (func &optional addr)
+  (let ((addr (downcase (or addr (mew-header-parse-address mew-from:) ""))))
+    ;; if From: is my address, addr is the list extracted from To:, Cc:
+    (if (and mew-refile-guess-from-me-is-special
+	     (mew-is-my-address (mew-get-my-address-regex-list) addr))
+	(let ((addr (mew-header-parse-address-list mew-refile-guess-key-list))
+	      (a nil) (r nil))
+	  (while addr
+	    (if (setq a (funcall func (car addr)))
+		(setq r (cons a r)))
+	    (setq addr (cdr addr)))
+	  (mew-uniq-list (nreverse r)))
+      (funcall func addr))))
 
 ;;
 ;; by default returns: guess1
 ;;
 (defun mew-refile-guess-by-default (&optional addr)
-  (let ((from (or addr (mew-header-parse-address mew-from:) "")))
-    (if (and mew-folders-default-folder
-	     (not (equal "" mew-folders-default-folder)))
-	(concat (file-name-as-directory mew-folders-default-folder)
-		(downcase (mew-addrstr-extract-user from)))
-      (concat "+" (downcase (mew-addrstr-extract-user from))))))
+  (mew-refile-guess-from-dispatch 'mew-refile-guess-by-default-body addr))
+
+(defun mew-refile-guess-by-default-body (&optional addr)
+  (let* ((pfix (cond ((equal mew-folders-default-folder "" ) "+")
+		     ((equal mew-folders-default-folder nil) "+")
+		     ((equal mew-folders-default-folder "+") "+")
+		     (t (file-name-as-directory mew-folders-default-folder))))
+	 (from (downcase (or addr (mew-header-parse-address mew-from:) "")))
+	 (user (mew-addrstr-extract-user from)))
+    (if mew-refile-guess-strip-domainpart
+	(concat pfix user)
+      (concat pfix from))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -315,19 +331,17 @@ any alredy marked message.")
       (set-buffer buf)
       (if (member 'mew-refile-guess-by-from mew-refile-guess-control)
 	  (mew-refile-guess-by-from-learn chosen info))
-      (if (member 'mew-refile-guess-by-msgid mew-refile-guess-control)
-	  (mew-refile-guess-by-msgid-learn chosen info))
-      (mew-refile-treat-alias-insert (mew-header-parse-address mew-from:)))))
-
+      (if (member 'mew-refile-guess-by-thread mew-refile-guess-control)
+	  (mew-refile-guess-by-thread-learn chosen info)))))
 ;;
 ;; learn from msgid
 ;;
-(defun mew-refile-guess-by-msgid-learn (chosen info)
+(defun mew-refile-guess-by-thread-learn (chosen info)
   (let* ((msgid  (mew-header-get-value mew-message-id:)) 
 	 (folder (car chosen))
 	 ;; ohter people's honest opinion and my honest opinion.
-	 (oho    (apply 'append info))
-	 (mho    (cdr (assoc 'mew-refile-guess-by-msgid info))))
+	 (oho    info)
+	 (mho    (cdr (assoc 'mew-refile-guess-by-thread info))))
     (if (and msgid (string-match "<[^ \n>]*>" msgid))
 	(setq msgid (substring msgid (match-beginning 0) (match-end 0))))
     (if (or (not msgid) (not chosen))
@@ -338,192 +352,88 @@ any alredy marked message.")
       (catch 'match
 	(while chosen
 	  (if (or (member (car chosen) mho)
-		  (not (member (car chosen) oho)))
+		  (not (catch 'find
+		    (while oho
+		      (and (member (car chosen) (car oho)) (throw 'find t))
+		      (setq oho (cdr oho))))))
 	      (throw 'match (setq folder (car chosen))))
 	  (setq chosen (cdr chosen))))
       (setq mew-refile-msgid-alist
 	    (cons (list msgid folder "??")
-		  (mew-refile-alist-purge msgid mew-refile-msgid-alist))))))
-
+		  (delete (assoc msgid mew-refile-msgid-alist)
+			  mew-refile-msgid-alist))))))
 ;;
 ;; learn from "From:" field
 ;;
 (defun mew-refile-guess-by-from-learn (chosen info)
-  (let ((from (mew-header-parse-address mew-from:))
-	(folder nil)
-	;; ohter people's honest opinion and my honest opinion.
-	(oho    (apply 'append info))
-	(mho    (cdr (assoc 'mew-refile-guess-by-from info))))
-    (if (or (not from) (not chosen))
-	()
-      ;; if my opninion was right, I learn it.
-      ;; or a folder was not in other people's opinion,
-      ;; I accept it.
+  (let* ((from  (downcase (or (mew-header-parse-address mew-from:) "")))
+	 (folder nil)
+	 ;; 'my honest opinion' guessed by mew-refile-guess-by-from.
+	 (mho    (nth 1 (assoc 'mew-refile-guess-by-from       info))))
+
+    (if (or (or (null from) (null chosen)) ;; if from and/or chosen is empty
+	    (and mho (member mho chosen))) ;; or my opinion was right,
+	()                                 ;; do nothing.
+
+      ;; I decide which folder is most important among the user chosen
+      ;; folders. 
       (catch 'match
 	(while chosen
-	  (if (or (member (car chosen) mho)
-		  (not (member (car chosen) oho)))
+	  ;; searching a folder anyone couldn't predict.
+	  (if (not (mew-refile-guess-member-p (car chosen) info))
 	      (throw 'match (setq folder (car chosen))))
 	  (setq chosen (cdr chosen))))
-      (if  folder
-	  (progn
-	    (setq mew-refile-from-alist
-		  (mew-refile-alist-purge from mew-refile-from-alist))
-	    (if mew-refile-from-alist
-		(nconc mew-refile-from-alist (list (cons from folder)))
-	      (setq mew-refile-from-alist (list (cons from folder)))))))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Aliases
-;;;
-
-;;
-;; Address completion in Draft mode.
-;; It should not be here...
-;;
-(defun mew-refile-treat-alias-insert (address)
-  (if (and (stringp address) (string-match "@" address))
-      (let (user)
-	;; Add empty information ("nom@mew.org" . nil) if key is absent.
-	(if (assoc address mew-refile-from-alist)
-	    ()
-	  (if mew-refile-from-alist
-	      (nconc mew-refile-from-alist (list (cons address nil)))
-	    (setq mew-refile-from-alist (list (cons address nil)))))
-	;; also Add to mew-alias-alist
-	(setq user (mew-addrstr-extract-user address))
-	(if (assoc user mew-alias-alist)
-	    ()
-	  (if mew-alias-alist
-	      (nconc mew-alias-alist (list (cons user address)))
-	    (setq mew-alias-alist (list (cons user address))))))))
-
-;;
-;; append alias items made from from-alist to alias-alist
-;; in reverse order.
-;;
-
-(defun mew-refile-alist-append-alias (alias-alist)
-  ;; must be uniqfy for alias completion
-  (let ((loop-alist mew-refile-from-alist) 
-	ret addr user)
-    (while loop-alist
-      (setq addr (car (car loop-alist)))
-      (setq user (mew-addrstr-extract-user addr))
-      (if (assoc user alias-alist)
-	  ()
-	(setq ret (cons (cons user addr) ret)))
-      (setq loop-alist (cdr loop-alist)))
-    (if alias-alist
-	(nconc alias-alist (nreverse ret))
-      (nreverse ret))))
-
-;;
-;; Alias maintenance function (purge)
-;;
-(defun mew-refile-delete-alias ()
-  "Delete an alias from mew-alias-alist and mew-refile-from-alist."
-  (interactive)
-  (let* ((addr (or (mew-delete-backward-char) ""))
-	 (user (mew-addrstr-extract-user addr))
-	 ent)
-    (insert addr)
-    (if (not (assoc addr mew-refile-from-alist))
-	(message "<%s> is not derived from auto learning." addr)
-      (if (not (y-or-n-p (format "Delete alias for %s ? " addr)))
-	  (message "...")
-	;; delete from alias-alist and from-alist
-	(setq mew-refile-from-alist
-	      (delete (assoc addr mew-refile-from-alist) mew-refile-from-alist))
-	(setq ent (assoc user mew-alias-alist))
-	(if (equal addr (cdr ent))
-	    (setq mew-alias-alist (delete ent mew-alias-alist)))
-	;; save from-alist for safety
-	(mew-refile-alist-save mew-refile-from-file-name mew-refile-from-alist)
-	;; delete address in screen.
-	(mew-delete-backward-char)
-	(message "Alias for %s is deleted." addr)))))
-;;
-;; Alias maintenance function (add)
-;;
-(defun mew-refile-add-alias ()
-  "Add an alias to mew-alias-alist and mew-refile-from-alist."
-  (interactive)
-  (let* ((addr (or (mew-delete-backward-char) ""))
-	 (key  (mew-addrstr-extract-user addr))
-	 value)
-    (insert addr)
-    (if (not (string-match "@" addr))
-	(message "<%s> is not complete address notation.")
-      (if (not (y-or-n-p (format "Add alias for %s ? " addr)))
-	  ()
-        ;; if key is absent, add empty information ("nom@mew.org" . nil).
-        ;; if key is already exists, move it to the top of the alist.
-	(if (setq value (assoc addr mew-refile-from-alist))
-	    (setq mew-refile-from-alist
-		  (cons value (delete value mew-refile-from-alist)))
+      ;; If candidate was found, I memorize it.
+      (if folder
 	  (setq mew-refile-from-alist
-		(cons (cons addr nil) mew-refile-from-alist)))
-	;; also Add to mew-alias-alist
-	(setq mew-alias-alist
-	      (cons 
-	       (cons key addr)
-	       (delete (assoc key mew-alias-alist) mew-alias-alist)))
-	;; save from-alist for safety
-	(mew-refile-alist-save mew-refile-from-file-name mew-refile-from-alist)
-	(message "Alias for %s is added." addr)))))
+		(cons (cons from folder)
+		      (delete (assoc from mew-refile-from-alist)
+			      mew-refile-from-alist)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; common routines for (a)list
+;;; Backyard functions.
 ;;;
 
-(defun mew-refile-alist-purge (key alist)
-  (let ((a alist))
-    (delete (assoc key a) a)))
-
-(defun mew-refile-alist-load (filename)
-  (let ((alist nil)
-	(fullname (expand-file-name filename mew-mail-path)))
-    (save-excursion
-      (if (not (file-readable-p fullname))
-	  ()
-	(mew-set-buffer-tmp)
-	(insert-file-contents fullname)
-	(setq alist 
-	      (condition-case nil
-		  (read (current-buffer))
-		(error ())))
-	alist))))
-
-(defun mew-refile-alist-save (filename alist)
-  (save-excursion
-    (let* ((fullname (expand-file-name filename mew-mail-path))
-	   (tmp-buf  (set-buffer (create-file-buffer fullname)))
-	   (n mew-refile-alist-max-length)
-	   (pointer (cons nil alist)))
-      (while pointer
-	(if (> n 0)
-	    (progn
-	      (setq pointer (cdr pointer))
-	      (setq n (1- n)))
-	  (setcdr alist nil)
-	  (setq pointer nil)))
-      (mew-erase-buffer)
-      (prin1 alist tmp-buf)
-      (princ "\n" tmp-buf)
-      (write-region (point-min) (point-max) fullname nil 'no-msg)
-      (kill-buffer tmp-buf))))
-
 (defun mew-refile-guess-save ()
   (if (and mew-refile-from-alist
 	   (member 'mew-refile-guess-by-from mew-refile-guess-control))
-      (mew-refile-alist-save mew-refile-from-file-name mew-refile-from-alist))
+      (mew-lisp-save mew-refile-from-file-name mew-refile-from-alist))
   (if (and mew-refile-msgid-alist
-	   (member 'mew-refile-guess-by-msgid mew-refile-guess-control))
-      (mew-refile-alist-save mew-refile-msgid-file-name
-			     mew-refile-msgid-alist)))
+	   (member 'mew-refile-guess-by-thread mew-refile-guess-control))
+      (mew-lisp-save mew-refile-msgid-file-name mew-refile-msgid-alist)))
+
+(defun mew-refile-guess-subfolder-p (parent child)
+  (string-match 
+   (concat  "^" (regexp-quote (file-name-as-directory parent)))
+   child))
+
+;;
+;; search x in a nested list.
+;;
+(defun mew-refile-guess-member-p (x lst)
+  (catch 'found
+    (while lst
+      (cond 
+       ((listp (car lst))
+	(if (mew-refile-guess-member-p x (car lst))
+	    (throw 'found t)))
+       (t
+	(if (equal  x (car lst))
+	    (throw 'found t))))
+      (setq lst (cdr lst)))))
+
+;;
+;; grep with regex in a list of string. (first match)
+;;
+(defun mew-refile-match-in-list (regex lst)
+  (let ((case-fold-search t))
+    (catch 'found 
+      (while lst
+	(if (string-match regex (car lst))
+	    (throw 'found (car lst)))
+	(setq lst (cdr lst))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -535,7 +445,7 @@ any alredy marked message.")
     (if folders
 	(if (not (mew-folder-member folder folders))
 	    (setq mew-summary-buffer-refile 
-		  (cons (append (list msg) folders (list folder))
+		  (cons (nreverse (cons folder (nreverse (cons msg folders))))
 			(delete msg-folders mew-summary-buffer-refile))))
       (setq mew-summary-buffer-refile 
 	    (cons (list msg folder) mew-summary-buffer-refile)))))
@@ -578,42 +488,60 @@ any alredy marked message.")
       (if (mew-folder-check (car folders))
 	  (setq ret (cons (car folders) ret)))
       (setq folders (cdr folders)))
-    (cons (nreverse ret) (cdr learn-info)) ;; return value
-    ))
+    (cons (nreverse ret) (cdr learn-info)))) ;; return value
 
-(defun mew-summary-refile (&optional exp-flds auto)
-  "Put the refile mark(default is 'o') on this message. 
-If already marked with 'o', it prints where this message 
-will be refiled. This can overlay other marks. When it overlays, 
-the cursor stays on the message. If it marks newly, displays 
-the next message."
-  (interactive)
+(defvar mew-header-reasonable-size 5000)
+
+(defun mew-summary-refile (&optional report)
+ "Put the refile mark(default is 'o') on this message. If already
+marked with 'o', it prints where this message will be refiled. This
+can overlay other marks. When it overlays, the cursor stays on the
+message. If it marks newly, displays the next message. If executed
+with '\\[universal-argument]', it displays how the refile rules work in Message mode."
+ (interactive "P")
+ (if report (mew-summary-refile-report) (mew-summary-refile-body)))
+
+(defun mew-summary-refile-body (&optional exp-flds auto)
   (mew-summary-only
    (mew-summary-msg-or-part
-    (let (msg folders mark buf learn-info)
+    (let (msg folders mark buf learn-info fld)
       (save-excursion
 	;; save the cursor position anyway
 	(mew-summary-goto-message)
 	;; on the message
+	(setq fld (mew-summary-folder-name))
 	(setq msg (mew-summary-message-number)) ;; msg is never nil
-	(setq mark (mew-summary-get-mark)) ;; any mark
-	(if exp-flds
-	    (setq folders exp-flds)
-	  ;; show message if not displayed
-	  (mew-summary-display t) ;; not force
+	(setq mark (mew-summary-get-mark))) ;; any mark
+      (if exp-flds
+	  (setq folders exp-flds)
+	;; show message if not displayed
+	(if (or auto (null mew-summary-buffer-disp-msg))
+	    (save-excursion
+	      (mew-set-buffer-tmp)
+	      (setq buf (current-buffer))
+	      (mew-insert-message fld msg mew-cs-autoconv
+				  mew-header-reasonable-size)
+	      (goto-char (point-min))
+	      (if (and (re-search-forward (concat "^$\\|^" mew-subj:) nil t)
+		       (not (looking-at "^$")))
+		  (let ((med (point)))
+		    (forward-line)
+		    (mew-header-goto-next)
+		    (mew-header-decode-region 'text med (point)))))
+	  (mew-summary-display nil) ;; cursor position is important
 	  (setq buf (or (mew-cache-hit (cons (buffer-name) msg))
-			;; non analysis
-			(mew-buffer-message)))
-	  (setq learn-info (mew-refile-decide-folders buf msg mark auto))
-	  (setq folders (car learn-info)))
- 	;; mark refile
-	(if (null folders)
-	    ()
-	  (or exp-flds (mew-refile-guess-learn buf learn-info))
-	  (mew-refile-reset msg)
-	  (mapcar (lambda (x) (mew-refile-set msg x)) folders)
-	  (mew-mark-unmark)
-	  (mew-summary-mark-as mew-mark-refile)))
+			(mew-buffer-message)))) ;; for ","
+	(setq learn-info (mew-refile-decide-folders buf msg mark auto))
+	(setq folders (car learn-info)))
+      ;; mark refile
+      (if folders
+	  (save-excursion
+	    (mew-summary-goto-message)
+	    (or exp-flds auto (mew-refile-guess-learn buf learn-info))
+	    (mew-refile-reset msg)
+	    (mapcar (function (lambda (x) (mew-refile-set msg x))) folders)
+	    (mew-mark-unmark)
+	    (mew-summary-mark-as mew-mark-refile)))
       ;; memorize last-folder
       (setq mew-refile-last-folder folders)
       (if (or mark auto (not folders))
@@ -625,53 +553,111 @@ the next message."
 	(beginning-of-line)
 	(let ((zmacs-regions nil))
 	  (push-mark (point) t t))
-	(mew-summary-display-next))
+	(mew-summary-display-after mew-summary-mark-direction))
       (set-buffer-modified-p nil)
       folders)))) ;; return value
 
+(defun mew-summary-refile-report ()
+  (mew-summary-only
+   (mew-summary-msg-or-part
+    (let (fld msg guess buf
+	      (customize-var '(mew-refile-ctrl-multi
+			       mew-refile-guess-key-list
+			       mew-refile-guess-strip-domainpart
+			       mew-refile-guess-from-me-is-special)))
+      (save-excursion
+	;; save the cursor position anyway
+	(mew-summary-goto-message)
+	;; on the message
+	(setq fld (mew-summary-folder-name))
+	(setq msg (mew-summary-message-number)) ;; msg is never nil
+	(mew-set-buffer-tmp)
+	(mew-insert-message fld msg mew-cs-autoconv
+			    mew-header-reasonable-size)
+	(setq guess (mew-refile-guess nil t)))
+      (setq buf (buffer-name))
+      (mew-window-configure buf 'message)
+      (mew-elet
+       (mew-erase-buffer)
+       (save-excursion
+	 ;; report result of guess.
+	 (insert (format "** Guess result: %s\n" (car guess)))
+	 ;; report status of customize variables.
+	 (insert "\n** Current Configurations:\n\n")
+	 (while customize-var
+	   (insert (format "%-40s:  " (car customize-var)))
+	   (insert (format "%s\n"     (eval (car customize-var))))
+	   (setq customize-var (cdr customize-var)))
+	 (insert "\n** Each function's opinion:\n\n")
+	 ;; report how each functions guessed.
+	 (setq guess (cdr guess))
+	 (while guess
+	   (insert (format "%-32s  " (car (car guess))))
+	   (insert (format "return: %s\n" (cdr (car guess))))
+	   (setq guess (cdr guess)))))
+      (mew-message-clear-end-of)
+      (set-buffer-modified-p nil)
+      (mew-pop-to-buffer buf)))))
+
 (defun mew-summary-refile-again ()
   "Put a refile mark on this message according to the previous 
 refile folder."
   (interactive)
   (mew-summary-only
-   (mew-summary-refile mew-refile-last-folder)))
+   (mew-summary-refile-body mew-refile-last-folder)))
 
-(defun mew-summary-auto-refile ()
-  "Refile each message in the folder automatically."
-  (interactive)
+(defun mew-summary-auto-refile (&optional mew-mark-review-only)
+  "Refile each message in the folder automatically. If 
+'mew-refile-auto-refile-skip-any-mark' is non-nil,
+any previousely marked message will be skipped.
+If '\\[universal-argument]' is specified, only messages marked with
+'mew-mark-review' will be conserned."
+  (interactive "P")
   (mew-summary-only
-   (let ((mew-analysis nil) 
-	 (after-change-function nil)
+   (let ((after-change-function nil)
 	 (after-change-functions nil)
 	 (mew-use-highlight-x-face nil)
 	 (lines (count-lines (point-min) (point-max)))
-	 (line 1)
-	 (mark nil))
-     (message "Auto refiling ...")
-     (save-window-excursion
-       (goto-char (point-min))
-       (while (not (eobp))
-	 (setq mark (mew-summary-get-mark))
-	 (or (equal mark mew-mark-refile)
-	     (equal mark mew-mark-delete)
-	     (and mark mew-refile-auto-refile-skip-any-mark)
-	     (mew-summary-refile nil t))
-	 (forward-line)
-	 (if (equal (% (/ (* 100 line) lines) 10) 0)
-	     (message "Auto refiling ... %s%%"
-		      (/ (* 100 line) lines)))
-	 (setq line (1+ line)))
-       (message "Auto refiling ... done")))))
+	 (line 1) (mark nil) msg)
+     (cond
+      (mew-mark-review-only
+       (setq msg (format "Refile all messages marked with '%c'? "
+			 mew-mark-review)))
+      (mew-refile-auto-refile-skip-any-mark
+       (setq msg (format "Refile all non-marked messages? ")))
+      (t
+       (setq msg (format "Refile all messages except those marked with '%c' and '%c'? " mew-mark-refile mew-mark-delete))))
+     (if (and mew-refile-auto-refile-confirm (null (yes-or-no-p msg)))
+	 (message "Not refiled.")
+       (message "Auto refiling ...")
+       (save-window-excursion
+	 (goto-char (point-min))
+	 (while (not (eobp))
+	   (setq mark (mew-summary-get-mark))
+	   (if mew-mark-review-only 
+	       (and (equal mark mew-mark-review)
+		    (mew-summary-refile-body nil t))
+	     (or (equal mark mew-mark-refile)
+		 (equal mark mew-mark-delete)
+		 (and mark mew-refile-auto-refile-skip-any-mark)
+		 (mew-summary-refile-body nil t)))
+	   (forward-line)
+	   (if (equal (% (/ (* 100 line) lines) 10) 0)
+	       (message "Auto refiling ... %s%%"
+			(/ (* 100 line) lines)))
+	   (setq line (1+ line)))
+	 (message "Auto refiling ... done"))))))
 
 ;;
 ;; "mx" extension
 ;;
 (defun mew-summary-mark-refile ()
-  "Refile messages marked with '*'."
+  "\\<mew-summary-mode-map>
+Put the refile mark onto all messages marked with '*'.
+This is very convenient to refile all messages picked by '\\[mew-summary-search-mark]'."
   (interactive)
   (mew-summary-only
-   (let ((mew-analysis nil)
-	 (after-change-function nil)
+   (let ((after-change-function nil)
 	 (after-change-functions nil)
 	 (mew-use-highlight-x-face nil)
 	 (last nil)
@@ -682,7 +668,7 @@ refile folder."
        (save-window-excursion
 	 (goto-char (point-min))
 	 (while (re-search-forward regex nil t)
-	   (setq last (mew-summary-refile last))
+	   (setq last (mew-summary-refile-body last))
 	   (forward-line))
 	 (message "Mark refiling ... done"))))))
 
@@ -690,7 +676,7 @@ refile folder."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994, 1995, 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
