@@ -58,29 +58,36 @@
   "List this folder asynchronously."
   (interactive "P")
   (mew-summary-only
-   (let* ((folder (buffer-name)) scanp range lines)
+   (let* ((folder (buffer-name))
+	  (askp mew-ask-range)
+	  (alist mew-range-auto-alist)
+	  scanp range lines)
      (mew-summary-folder-cache-manage folder)
-     (mew-mark-clean)
-     (if (or jump (mew-folder-imapp folder)) (goto-char (point-max)))
-     (mew-buffers-setup folder)
+     ;; check scanp before (mew-mark-clean)
      (cond
       ((interactive-p)
-       (setq range (mew-input-range folder mew-range-interactive-alist t))
+       (setq alist mew-range-interactive-alist)
+       (setq askp t)
        (setq scanp t))
       ((and mew-summary-cache-use mew-summary-imap-cache
 	    (not (member folder mew-summary-imap-nocache-folders))
 	    (mew-folder-imapp folder)
 	    (file-exists-p (mew-expand-folder folder mew-summary-cache-file)))
-       (setq range (mew-input-range folder mew-range-auto-alist mew-ask-range))
        (setq scanp t))
       ((mew-folder-remotep folder) ;; xxx
-       (setq range (mew-input-range folder mew-range-interactive-alist t))
+       (setq alist mew-range-interactive-alist)
+       (setq askp t)
        (setq scanp t))
       ((and mew-summary-cache-use (mew-summary-folder-dir-newp))
-       (setq range (mew-input-range folder mew-range-auto-alist mew-ask-range))
        (setq scanp t)))
+     ;;
+     (mew-mark-clean)
+     (set-buffer-modified-p nil)
+     (if (or jump (mew-folder-imapp folder)) (goto-char (point-max)))
+     (mew-buffers-setup folder)
      (if (not scanp)
 	 (or arg (goto-char (point-max)))
+       (setq range (mew-input-range folder alist askp))
        (or arg (goto-char (point-max)))
        (if (equal (car range) "all")
 	   (setq lines (mew-summary-mark-collect3 mew-mark-review)))
@@ -127,7 +134,8 @@
 	    (setq mew-summary-buffer-config mew-config-imget)
 	    (setq mew-summary-buffer-error nil)
 	    (setq mew-summary-buffer-wrong-pws nil)
-	    (mew-piolet read mew-cs-pick
+	    (mew-pioalet
+	     read mew-cs-pick mew-cs-pick
 	     (setq mew-summary-buffer-process
 		   (apply (function start-process) 
 			  prog;; name
@@ -311,10 +319,8 @@
 	;; On PPP environment, executing "imget" lets the dial be up.
 	;; So, it's a good idea to flush queue at this time
 	;; if messages to be sent exist.
-	(if (and mew-auto-flush-queue
-		 mew-queue-path (file-directory-p mew-queue-path)
-		 (> (length (directory-files mew-queue-path)) 2))
-	    (progn
+	(if (and mew-auto-flush-queue (mew-flushable-p))
+	    (let ((mew-ask-flush-queue nil))
 	      (sit-for 1)		; wait
 	      (mew-summary-flush-queue)))))
       (cond
@@ -359,37 +365,37 @@ Use this command to solve this problem."
 ;;; Summary file cache
 ;;;
 
-(defun mew-summary-folder-dir-newp ()
-  ;; buffer switched
+(defun mew-summary-compare-times (type)
   (let* ((dir (file-chase-links (mew-expand-folder (buffer-name))))
-	 (tdir (mew-file-get-time dir))
-	 (da (car tdir))
-	 (db (car (cdr tdir)))
+	 (tdir (if mew-touch-folder-p
+		   (mew-file-get-time
+		    (expand-file-name mew-summary-touch-file
+				      (mew-expand-folder dir)))
+		 (mew-file-get-time dir)))
 	 (cache (expand-file-name mew-summary-cache-file dir))
 	 (tcache (mew-file-get-time cache))
-	 (fa (car tcache))
-	 (fb (car (cdr tcache))))
+	 (tbuf mew-summary-buffer-folder-cache-time)
+	 t1 t2)
     (cond
-     ((null tdir) nil)
-     ((null tcache) t) ;; no cache, do update!
-     ((> da fa) t)
-     ((= da fa) (if (> db fb) t nil)) ;; nil if same 
+     ((eq type 'dir-cache)
+      (setq t1 tdir)
+      (setq t2 tcache))
+     ((eq type 'cache-buf)
+      (setq t1 tcache)
+      (setq t2 tbuf)))
+    (cond
+     ((null t1) nil)
+     ((null t2) t) ;; do update
+     ((> (nth 0 t1) (nth 0 t2)) t)
+     ((= (nth 0 t1) (nth 0 t2))
+      (if (> (nth 1 t1) (nth 1 t2)) t nil)) ;; nil if equal
      (t nil))))
 
-(defun mew-summary-folder-cache-oldp ()
-  ;; buffer switched
-  (let* ((file (mew-expand-folder (buffer-name) mew-summary-cache-file))
-	 (tfile (mew-file-get-time file))
-	 (fa (car tfile))
-	 (fb (car (cdr tfile)))
-	 (tbuf mew-summary-buffer-folder-cache-time)
-	 (ba (car tbuf))
-	 (bb (car (cdr tbuf))))
-    (cond
-     ((or (null tfile) (null tbuf)) nil)
-     ((> fa ba) t)
-     ((= fa ba) (if (> fb bb) t nil)) ;; nil if same 
-     (t nil))))
+(defmacro mew-summary-folder-dir-newp ()
+  '(mew-summary-compare-times 'dir-cache))
+
+(defmacro mew-summary-folder-cache-newp ()
+  '(mew-summary-compare-times 'cache-buf))
 
 (defmacro mew-summary-folder-cache-updatep (folder)
   (` (and mew-summary-cache-use
@@ -403,11 +409,12 @@ Use this command to solve this problem."
   (if (mew-summary-folder-cache-updatep folder)
       (let ((cache (mew-expand-folder folder mew-summary-cache-file)))
         (if (and (file-exists-p cache)
-		 (or (mew-summary-folder-cache-oldp) (mew-folder-imapp folder)))
+		 (or (mew-summary-folder-cache-newp) (mew-folder-imapp folder)))
 	    (mew-elet
 	     (mew-erase-buffer)
-	     (mew-frwlet mew-cs-scan mew-cs-dummy
-			 (insert-file-contents cache))
+	     (mew-frwlet
+	      mew-cs-scan mew-cs-dummy
+	      (insert-file-contents cache))
 	     (setq mew-summary-buffer-folder-cache-time 
 		   (mew-file-get-time cache))
 	     (mew-summary-batch-unmark (list mew-mark-refile) nil)
@@ -430,11 +437,13 @@ Use this command to solve this problem."
 		    (mew-set-buffer-tmp)
 		    (insert-buffer-substring cbuf min beg)
 		    (insert-buffer-substring cbuf end max)
-		    (mew-frwlet mew-cs-dummy mew-cs-scan
-		      (write-region (point-min) (point-max) cache nil 'no-msg))
+		    (mew-frwlet
+		     mew-cs-dummy mew-cs-scan
+		     (write-region (point-min) (point-max) cache nil 'no-msg))
 		    (set-buffer cbuf))
-		(mew-frwlet mew-cs-dummy mew-cs-scan
-		  (write-region (point-min) (point-max) cache nil 'no-msg)))
+		(mew-frwlet
+		 mew-cs-dummy mew-cs-scan
+		 (write-region (point-min) (point-max) cache nil 'no-msg)))
 	      (setq mew-summary-buffer-folder-cache-time
 		    (mew-file-get-time cache)))))))
 
