@@ -2,11 +2,11 @@
 
 ;; Author:  Shuichi Kitaguchi <kit@Mew.org>
 ;; Created: Dec 05, 1997
-;; Revised: Aug 30, 1999
+;; Revised: Nov 20, 1999
 
 ;;; Code:
 
-(defconst mew-win32-version "mew-win32.el 0.33")
+(defconst mew-win32-version "mew-win32.el 0.33.1")
 
 ;;; for NTEmacs User
 ;;
@@ -17,6 +17,17 @@
 ;;  ~/.emacs settings
 ;;
 
+;;; for PGP/MIME
+;;    mew-prog-pgp           ...  executable name for PGP/MIME
+;;
+;;; example
+;;    (setq mew-prog-pgp  "pgp.exe")      ;; PGP
+;;    (setq mew-prog-pgp  "gpg.exe")      ;; GNUPG
+
+;;; for Windows NT with NTFS
+;;(eval-after-load "mew-win32"
+;;  '(setq mew-touch-folder-p nil))
+
 ;;; for PRINTING
 ;;    mew-w32-prog-print     ...  print command
 ;;    mew-w32-prog-print-arg ...  print command argument
@@ -31,6 +42,7 @@
 ;;   (define-process-argument-editing "/notepad\\.exe$"
 ;;     (lambda (x)
 ;;       (general-process-argument-editing-function x nil t)))
+;;   (setq mew-print-function (function mew-w32-print-buffer))
 
 ;; Win32 programs.
 (defvar mew-w32-exec           "fiber.exe")
@@ -100,9 +112,15 @@
 (setq mew-prog-gzip         "gzip.exe")
 (setq mew-prog-shell        shell-file-name)
 
-(setq mew-prog-pgp          "pgp.exe")
+;(setq mew-prog-pgp          "pgp.exe")
+(setq mew-prog-pgp2         "pgp.exe")
+(setq mew-prog-pgp5         "pgp.exe")
+(setq mew-prog-pgp5e        "pgpe.exe")
+(setq mew-prog-pgp5s        "pgps.exe")
+(setq mew-prog-pgp5v        "pgpv.exe")
+(setq mew-prog-pgp5k        "pgpk.exe")
+(setq mew-prog-gpg          "gpg.exe")
 
-(setq mew-prog-utime        "utime.exe")
 (setq mew-touch-folder-p    t)
 (setq mew-delete-temp-file  nil)
 
@@ -118,8 +136,9 @@
 (defun mew-w32-print-buffer ()
   (interactive)
   (setq tempfile (mew-make-temp-name))
-  (mew-frwlet mew-cs-dummy mew-w32-cs-print
-    (write-region (point-min) (point-max) tempfile nil nil))
+  (mew-frwlet
+   mew-cs-dummy mew-w32-cs-print
+   (write-region (point-min) (point-max) tempfile nil nil))
   (setq w32-start-process-show-window t)
   (cond
    ((eq mew-w32-prog-print-arg nil)
@@ -128,7 +147,6 @@
     (call-process mew-w32-prog-print nil nil nil mew-w32-prog-print-arg tempfile)))
   (setq w32-start-process-show-window nil)
   (delete-file tempfile))
-(setq mew-print-function (function mew-w32-print-buffer))
 
 
 ;; MIME setting
