@@ -2,7 +2,7 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1994
-;; Revised: Oct 15, 1999
+;; Revised: Feb 28, 1999
 
 ;;; Commentary:
 
@@ -35,7 +35,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "1.94.1"
+(defconst mew-version-number "1.94.2"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -456,7 +456,7 @@ folders is stored in '~/Mail/.folders'. The default value is 't'."
 	(mew-member-case-equal 
 	 (cdr (assq
 	       'window-id
-	       (frame-parameters (window-frame (selected-window)))))
+	       (frame-parameters (selected-frame))))
 	 (sort
 	  (mapcar (function (lambda (frame)
 			      (cdr (assq 'window-id
@@ -588,7 +588,12 @@ It is typically called by kill-emacs-hook."
     (if (and (eq major-mode 'mew-summary-mode)
 	     (mew-folder-remotep folder))
 	(mew-remote-folder-cache-delete folder))
-    (if (get-buffer buf) (kill-buffer buf))))
+    (if (get-buffer buf)
+	(progn
+	  (save-excursion
+	    (set-buffer buf)
+	    (mew-overlay-delete-buffer))
+	  (kill-buffer buf)))))
 
 (defun mew-buffer-message-clean-up (func)
   (if window-system
@@ -618,6 +623,11 @@ It is typically called by kill-emacs-hook."
 			major-mode)))
 	  (mew-kill-buffer b)))))
 
+
+(defmacro mew-quit-toolbar-update ()
+  '(if (fboundp 'redraw-frame) ;; for BOW
+       (redraw-frame (selected-frame)))) ;; update toolbar
+
 (defun mew-summary-suspend ()
   "Suspend Mew then switch to another buffer. All buffers of 
 Mew remain, so you can resume with buffer operations."
@@ -625,7 +635,7 @@ Mew remain, so you can resume with buffer operations."
   (mew-buffer-message-clean-up (function mew-bury-buffer))
   (mew-buffers-bury)
   (mew-window-pop)
-  (redraw-frame (window-frame (selected-window))) ;; update toolbar
+  (mew-quit-toolbar-update)
   (run-hooks 'mew-suspend-hook))
 
 (defun mew-summary-quit ()
@@ -654,7 +664,7 @@ Mew remain, so you can resume with buffer operations."
     ;;
     (mew-window-pop)
     (mew-window-clean-up)
-    (redraw-frame (window-frame (selected-window))) ;; update toolbar
+    (mew-quit-toolbar-update)
     ;;
     (mew-config-clean-up) ;; MUST be last
     ;; flush minibuffer
